

class AgentlessItem(object):
    def __init__(self, hash=None, script=None, name = None, user=None, uid=None, gid=None):
        self.hash = hash
        self.script = script
        self.name = name
        self.user = user
        self.uid = uid
        self.gid = gid