# Generated by h2py from /usr/include/netinet/in.h

# Included from sys/stdsyms.h
_POSIX_C_SOURCE = 2
_FILE_OFFSET_BITS = 64
_LFS_LARGEFILE = 1
_LFS64_LARGEFILE = 1
def __(arg): return arg

def __(arg): return ()

_INCLUDE_HPUX_API_LEVEL = 20040821
_INCLUDE_HPUX_API_LEVEL = 0
_INCLUDE_HPUX_API_LEVEL = 0

# Included from sys/types.h

# Included from sys/_inttypes.h

# Included from machine/vm/vmtypes.h
MAXSUSE = 65535

# Included from sys/_fd_macros.h
_MAXFUPLIM = (1024 * 1024)
_MAXFUPLIM = 60000
_MAXFUPLIM = 2048
FD_SETSIZE = _MAXFUPLIM
_MAXFUPLIM = (1024 * 1024)
FD_SETSIZE = _MAXFUPLIM
MAXFUPLIM = _MAXFUPLIM

# Included from .unsupp/sys/_types.h

# Included from machine/sys/_types.h
IPPROTO_IP = 0
IPPROTO_ICMP = 1
IPPROTO_IGMP = 2
IPPROTO_GGP = 3
IPPROTO_IPIP = 4
IPPROTO_TCP = 6
IPPROTO_EGP = 8
IPPROTO_IGP = 9
IPPROTO_PUP = 12
IPPROTO_UDP = 17
IPPROTO_IDP = 22
IPPROTO_XTP = 36
IPPROTO_ESP = 50
IPPROTO_AH = 51
IPPROTO_OSPF = 89
IPPROTO_IPENCAP = 94
IPPROTO_ENCAP = 98
IPPROTO_IPPCP = 108
IPPROTO_PXP = 241
IPPROTO_RAW = 255
IPPROTO_MAX = 256
IPPORT_RESERVED = 1024
IPPORT_USERRESERVED = 5000
IMPLINK_IP = 155
IMPLINK_LOWEXPER = 156
IMPLINK_HIGHEXPER = 158
INET_ADDRSTRLEN = 16
IN_CLASSA_NSHIFT = 24
IN_CLASSA_MAX = 128
IN_CLASSB_NSHIFT = 16
IN_CLASSB_MAX = 65536
IN_CLASSC_NSHIFT = 8
IN_CLASSD_NSHIFT = 28
def IN_MULTICAST(i): return IN_CLASSD(i)

IN_LOOPBACKNET = 127
def IN_CLASSA_BROADCAST(i): return \

def IN_CLASSB_BROADCAST(i): return \

def IN_CLASSC_BROADCAST(i): return \

def IN_BROADCAST(i): return \

def IN_LOOPBACK(i): return \

def IN_PRIVATE(i): return \

def ntohl(x): return (x)

def ntohs(x): return (x)

def htonl(x): return (x)

def htons(x): return (x)

IP_OPTIONS = 1
IP_MULTICAST_IF = 2
IP_MULTICAST_TTL = 3
IP_MULTICAST_LOOP = 4
IP_ADD_MEMBERSHIP = 5
IP_DROP_MEMBERSHIP = 6
IP_TOS = 8
IP_TTL = 16
IP_HDRINCL = 0x1002
IP_RECVOPTS = 0x1005
IP_RECVRETOPTS = 0x1006
IP_RECVDSTADDR = 0x1007
IP_RETOPTS = 0x1008
IP_BROADCAST_IFNAME = 0x1015
IP_RECVIFADDR = 0x1016
IP_LINK_STATUS = 0x1017
IP_BROADCAST_IF = 0x1018
IP_RECVLINKSRC = 0x1019
IP_RECVTTL = 0x1020
IP_TUNNEL_NOTIFIES = 0x1021
IP_DEFAULT_MULTICAST_TTL = 1
IP_DEFAULT_MULTICAST_LOOP = 1
IP_MAX_MEMBERSHIPS = 20

# Included from netinet/in6.h
from IN import *
IPPROTO_HOPOPTS = 0
IPPROTO_IPV6 = 41
IPPROTO_ROUTING = 43
IPPROTO_FRAGMENT = 44
IPPROTO_ICMPV6 = 58
IPPROTO_NONE = 59
IPPROTO_DSTOPTS = 60
IPPROTO_MIP6 = 135
IPV6_UNICAST_HOPS = 1
IPV6_MULTICAST_IF = 2
IPV6_MULTICAST_HOPS = 3
IPV6_MULTICAST_LOOP = 4
IPV6_ADD_MEMBERSHIP = 5
IPV6_DROP_MEMBERSHIP = 6
IPV6_CHECKSUM = 7
IPV6_PKTINFO = 8
IPV6_HOPLIMIT = 9
IPV6_DSTOPTS = 10
IPV6_HOPOPTS = 11
IPV6_RTHDR = 12
IPV6_RECVPKTINFO = 13
IPV6_RECVHOPLIMIT = 14
IPV6_RECVDSTOPTS = 15
IPV6_RECVHOPOPTS = 16
IPV6_RECVRTHDR = 17
IPV6_LINK_STATUS = 18
IPV6_RTHDRDSTOPTS = 19
IPV6_JOIN_GROUP = 20
IPV6_LEAVE_GROUP = 21
IPV6_NEXTHOP = 22
IPV6_RECVRTHDRDSTOPTS = 23
IPV6_PATHMTU = 24
IPV6_RECVPATHMTU = 25
IPV6_REACHCONF = 26
IPV6_USE_MIN_MTU = 27
IPV6_V6ONLY = 28
IPV6_BINDING_BYPASS = 29
IPV6_BINDING_TUNNEL = 30
IPV6_IPSECINFO = 31
IPV6_RECVIPSECINFO = 32
IPV6_DEFAULT_MULTICAST_HOPS = 1
IPV6_DEFAULT_MULTICAST_LOOP = 1
IPV6_RTHDR_TYPE_0 = 0
INET6_ADDRSTRLEN = 46
IN6ADDR_ANY_INIT = \
                         { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
IN6ADDR_LOOPBACK_INIT = \
        	         { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 }
def IN6_IS_ADDR_UNSPECIFIED(i): return \

def IN6_IS_ADDR_LOOPBACK(i): return \

def IN6_IS_ADDR_V4COMPAT(i): return \

def IN6_IS_ADDR_MC_NODELOCAL(i): return \

def IN6_IS_ADDR_MC_LINKLOCAL(i): return \

def IN6_IS_ADDR_MC_SITELOCAL(i): return \

def IN6_IS_ADDR_MC_ORGLOCAL(i): return \

def IN6_IS_ADDR_MC_GLOBAL(i): return \

